Index: sys/dev/flash/mx25l.c
===================================================================
--- sys/dev/flash/mx25l.c	(revision 253747)
+++ sys/dev/flash/mx25l.c	(working copy)
@@ -43,6 +43,14 @@
 #include <dev/spibus/spi.h>
 #include "spibus_if.h"
 
+#include "opt_platform.h"
+
+#ifdef FDT
+#include <dev/fdt/fdt_common.h>
+#include <dev/ofw/ofw_bus_subr.h>
+#include <dev/ofw/openfirm.h>
+#endif
+
 #include <dev/flash/mx25lreg.h>
 
 #define	FL_NONE			0x00
@@ -76,6 +84,7 @@
 	struct disk	*sc_disk;
 	struct proc	*sc_p;
 	struct bio_queue_head sc_bio_queue;
+	struct intr_config_hook sc_intr_hook;
 	unsigned int	sc_flags;
 };
 
@@ -358,23 +367,27 @@
 static int
 mx25l_probe(device_t dev)
 {
+
+#ifdef FDT
+	if (!ofw_bus_is_compatible(dev, "flash,mx25l"))
+		return (ENXIO);
+#endif
 	device_set_desc(dev, "M25Pxx Flash Family");
 	return (0);
 }
 
-static int
-mx25l_attach(device_t dev)
+static void
+mx25l_start(void *arg)
 {
 	struct mx25l_softc *sc;
 	struct mx25l_flash_ident *ident;
 
-	sc = device_get_softc(dev);
-	sc->sc_dev = dev;
-	M25PXX_LOCK_INIT(sc);
+	sc = (struct mx25l_softc *)arg;
 
+	config_intrhook_disestablish(&sc->sc_intr_hook);
 	ident = mx25l_get_device_ident(sc);
 	if (ident == NULL)
-		return (ENXIO);
+		return;
 
 	mx25l_wait_for_device_ready(sc->sc_dev);
 
@@ -403,7 +416,27 @@
 	kproc_create(&mx25l_task, sc, &sc->sc_p, 0, 0, "task: mx25l flash");
 	device_printf(sc->sc_dev, "%s, sector %d bytes, %d sectors\n", 
 	    ident->name, ident->sectorsize, ident->sectorcount);
+}
 
+static int
+mx25l_attach(device_t dev)
+{
+	struct mx25l_softc *sc;
+
+	sc = device_get_softc(dev);
+	sc->sc_dev = dev;
+	M25PXX_LOCK_INIT(sc);
+
+	sc->sc_intr_hook.ich_func = mx25l_start;
+	sc->sc_intr_hook.ich_arg = sc;
+
+	/*
+	 * We have to wait until interrupts are enabled.  In some cases SPI
+	 * read and write only works if the interrupts are available.
+	 */
+	if (config_intrhook_establish(&sc->sc_intr_hook) != 0)
+		return (ENOMEM);
+
 	return (0);
 }
 
